#property copyright "Copyright © 2017-2018, Marketeer"
#property link      "https://www.mql5.com/en/users/marketeer"
#property version   "1.0"
#property description "Custom Symbol Testbed\n"
#property description "Builds custom M1 chart from current work chart, which is supposed to be H1, M30, M15, etc (not M1)."


#define SUPPRESS_BARS
#include <ind4to5.mqh>

#include <Symbol.mqh>

input bool EmulateOnLineChart = true;
input bool Reset = false;

//+------------------------------------------------------------------+
string sSymbolName;
bool bStopAll = false;
int iRcdCnt = 0;

datetime prevTime;
datetime lastQuoteTime;
bool firstRun;
bool justCreated;

//+------------------------------------------------------------------+

void updateChartWindow()
{
  if(EmulateOnLineChart)
  {
    MqlTick tick[1];
    SymbolInfoTick(_Symbol, tick[0]);
    tick[0].time = prevTime;
    tick[0].time_msc = prevTime * 1000;

    int added = CustomTicksAdd(sSymbolName, tick);
    if(added == -1)
    {
      Print("CustomTicksAdd failed: ", GetLastError(), " / Last time:", TimeToString(prevTime, TIME_DATE|TIME_MINUTES|TIME_SECONDS));
      if(GetLastError() == ERR_MARKET_UNKNOWN_SYMBOL)
      {
        bStopAll = true;
        Alert("Custom symbol ", sSymbolName, " is unavailable, expert is disabled");
      }
      else
      if(GetLastError() == ERR_MARKET_NOT_SELECTED)
      {
        bStopAll = true;
        Alert("Custom symbol ", sSymbolName, " is not selected, expert is disabled");
      }
    }
  }
}

//+------------------------------------------------------------------+

MqlRates firstBar[1] = {0};
MqlRates ending[3] = {0};


bool checkOtherInstance()
{
  long me = ChartID();
  string signature = ChartSymbol() + EnumToString(ChartPeriod());
  string ea = ChartGetString(0, CHART_EXPERT_NAME);
  long id = ChartFirst();
  while(id != -1)
  {
    if(id != me)
    {
      string name = ChartGetString(id, CHART_EXPERT_NAME);
      if(name == ea)
      {
        if(signature == ChartSymbol(id) + EnumToString(ChartPeriod(id)))
        {
          return false;
        }
      }
    }
    id = ChartNext(id);
  }
  return true;
}

int OnInit(void)
{
  bStopAll = false;
  iRcdCnt = 0;
  justCreated = false;
  
  if(SymbolInfoInteger(_Symbol, SYMBOL_CUSTOM))
  {
    Alert("" + _Symbol + " is a custom symbol. Only built-in symbol can be used as a host.");
    return INIT_FAILED;
  }
  
  if(!checkOtherInstance())
  {
    if(IDCANCEL == MessageBox("Other instance of EA is already running on the same symbol/timeframe. Are you sure?", "", MB_OKCANCEL))
    {
      return INIT_FAILED;
    }
  }
  
  string p = EnumToString(Period());
  StringReplace(p, "PERIOD", "");
  sSymbolName = Symbol() + p + "_TB";

  if(!SymbolSelect(sSymbolName, true))
  {
    Comment("Creating custom symbol ", sSymbolName, ", please wait...");
    Print("Creating custom symbol ", sSymbolName, ", please wait...");
    const SYMBOL Symb(sSymbolName);
    Symb.Template(_Symbol);
    justCreated = true;
    
    if(!SymbolSelect(sSymbolName, true))
    {
      Alert("Can't select symbol:", sSymbolName, " err:", GetLastError());
      return INIT_FAILED;
    }
  }
  else
  {
    Comment("Updating custom symbol ", sSymbolName, ", please wait...");
    Print("Updating custom symbol ", sSymbolName, ", please wait...");
  }
  
  lastQuoteTime = 0;

  int n = CopyRates(sSymbolName, PERIOD_M1, 0, 3, ending);
  if(n == 3)
  {
    lastQuoteTime = ending[2].time; // ending[2] is incomplete bar
    // search for first lastQuoteTime
    datetime rawBarTime[1];
    // find symbol/period bar which contains incomplete data
    if(CopyTime(_Symbol, PERIOD_CURRENT, lastQuoteTime, 1, rawBarTime) == 1)
    {
      Print("Last CopyBar:", rawBarTime[0], "->", lastQuoteTime);
      lastQuoteTime = rawBarTime[0];
    }
    else
    {
      Print("Last CopyTime:", lastQuoteTime);
    }
    ArrayPrint(ending);
    
    if(CustomRatesDelete(sSymbolName, lastQuoteTime, LONG_MAX) == -1)
    {
      Print("CustomRatesDelete failed:", GetLastError());
    }
    else
    {
      n = CopyRates(sSymbolName, PERIOD_M1, 0, 3, ending);
      if(n == 3)
      {
        ArrayPrint(ending);
      }
    }
  }
  else
  {
    Print("CopyRates failed, code: ", GetLastError());
  }
  
  int b = Bars(sSymbolName, PERIOD_M1);
  Print("Found ", b, " bars");
  
  n = CopyRates(sSymbolName, PERIOD_M1, b - 1, 1, firstBar);
  if(n > 0)
  {
    Print("First bar in ", sSymbolName, ": ", firstBar[0].time);
  }
  
  EventSetTimer(1);
  firstRun = true;
  
  return INIT_SUCCEEDED;
}

void OnTimer()
{
  OnTick();
}

void OnTick(void)
{
  if(bStopAll) return;

  //+------------------------------------------------------------------+
  // This is only executed once, then the first tick arrives.
  if(firstRun)
  {
    // Error checking	
    if(!TerminalInfoInteger(TERMINAL_CONNECTED))
    {
      Print("Waiting for connection...");
      return;
    }
    
    if(!SeriesInfoInteger(_Symbol, _Period, SERIES_SYNCHRONIZED))
    {
      Print("Unsynchronized, skipping ticks...");
      return;
    }
    
    EventKillTimer();
    Comment("Processing...");
    
    if(Reset || (lastQuoteTime == 0))
    {
      Print("Resetting range ", firstBar[0].time, " - ", lastQuoteTime);
      CustomSymbolSetInteger(sSymbolName, SYMBOL_TRADE_MODE, SYMBOL_TRADE_MODE_DISABLED);
      ResetLastError();
      int deleted = CustomRatesDelete(sSymbolName, 0, LONG_MAX);
      int err = GetLastError();
      if(err != ERR_SUCCESS)
      {
        Alert("CustomRatesDelete at ", lastQuoteTime, " failed, ", err, ", please, restart the expert!");
        bStopAll = true;
        return;
      }
      else
      {
        Print("Rates deleted: ", deleted);
      }
      
      ResetLastError();
      deleted = CustomTicksDelete(sSymbolName, 0, LONG_MAX);
      if(deleted == -1)
      {
        Print("CustomTicksDelete failed ", GetLastError());
      }
      else
      {
        Print("Ticks deleted: ", deleted);
      }
    
      lastQuoteTime = 0;
    }
    
    // process historical data
    int i = Bars(_Symbol, _Period) - 2;

    string ss[];
    if(StringSplit(EnumToString(_Period), '_', ss) == 2)
    {
      Print("First bar on ", _Symbol, " ", ss[1], " ", Time[i + 1]);
    }

    // if this is a continuation, adjust it so that it points to a bar for latest data
    if(lastQuoteTime > 0)
    {
      // find base symbol/period bar where latest existing data maps
      i = iBarShift(_Symbol, PERIOD_CURRENT, lastQuoteTime);
      
      // iBarShift is not reliable, so we can get next bar
      // actually we need to run a loop until Time[i] is larger than latest time
      while(Time[i] > lastQuoteTime && i < Bars(_Symbol, _Period) - 1)
      {
        Print("Correction:", i, " ", Time[i], "->", Time[i + 1]);
        i++;
      }
      prevTime = ending[2].time;
      Print("Restarted from ", Time[i], " [", i, "] ", prevTime);
    }
    else
    {
      Print("Starting from beginnig");
      prevTime = Time[i + 1];
    }
    
    static uint trap = 0;

    //---------------------------------------------------------------------------
    while(i >= 0)
    {
      doWriteStruct(Time[i], Open[i], High[i], Low[i], Close[i], Volume[i]);

      if(GetTickCount() - trap > 1000)
      {
        Comment("Processing bar: ", i, " / ", (Bars(_Symbol, _Period) - 1));
        Print("Processing bar: ", i, " / ", (Bars(_Symbol, _Period) - 1));
        ChartRedraw();
        trap = GetTickCount();
      }
      
      i--;
    }
    trap = 0;
    
    CustomSymbolSetInteger(sSymbolName, SYMBOL_TRADE_MODE, SymbolInfoInteger(_Symbol, SYMBOL_TRADE_MODE));
    
    firstRun = false;

    if(bStopAll) return;
    
    Comment("TestBed: open ", sSymbolName, " / ", iRcdCnt, " bars");
    
    if(justCreated)
    {
      long id = ChartOpen(sSymbolName, PERIOD_M1);
      if(id == 0)
      {
        Alert("Can't open new chart for " + sSymbolName + ", code: " + (string)GetLastError());
      }
      else
      {
        Sleep(1000);
        ChartSetSymbolPeriod(id, sSymbolName, PERIOD_M1);
        ChartSetInteger(id, CHART_MODE, CHART_CANDLES);
      }
    }
    return;
    
  } // end firstRun
  
  //----------------------------------------------------------------------------
  // we always enter here after history done
  // Begin live data feed
  if(bStopAll) return;
  
  doWriteStruct(Time[0], Open[0], High[0], Low[0], Close[0], Volume[0]);

  if(!EmulateOnLineChart) return;
  
  updateChartWindow();
  
}

//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
  Comment("");
  return;
}

//+------------------------------------------------------------------+
void doWriteStruct(const datetime dtTime, const double dOpen, const double dHigh, const double dLow, const double dClose, const double dVol)
{
  iRcdCnt++;

  MqlRates rate[1];
  rate[0].time = dtTime;
  rate[0].open = dOpen;
  rate[0].high = dHigh;
  rate[0].low = dLow;
  rate[0].close = dClose;
  rate[0].tick_volume = (long) dVol;
  rate[0].spread = 0;
  rate[0].real_volume = (long) dVol;

  prevTime = dtTime;

  if(dVol == 0)
  {
    if(rate[0].tick_volume == 0) rate[0].tick_volume = 1;
    if(rate[0].real_volume == 0) rate[0].real_volume = 1;
  }
  
  if(CustomRatesUpdate(sSymbolName, rate) <= 0)
  {
    Alert("Error on writing custom record: " + (string)iRcdCnt + ", err: " + (string)GetLastError());
    bStopAll = true;
  }
}

//+------------------------------------------------------------------+
